module arb_avalon_interface(
	/* verilator lint_off UNUSEDSIGNAL */
	input logic 		clk,
	input logic 		reset,
	/* verilator lint_on UNUSEDSIGNAL */
	input logic			read,
	input logic			write,
	input logic			chipselect,

	// one wider than inner module to allow dummy 0th address for reading registers
	input logic [8:0]	address,
	input logic [31:0]	writedata,

	output logic [31:0]	readdata
);

	logic arb_start;
	logic arb_write;
	logic arb_read;
	logic [31:0] arb_writedata;
	logic [7:0] arb_readaddr;
	logic arb_done;
	logic [7:0] arb_resultcount;
	logic [31:0] arb_readdata;
	logic arb_reset;

	assign arb_read 			= read && chipselect && address != 0;
	assign arb_write			= write && chipselect && address != 0;
	assign arb_writedata		= writedata; // contents of writedata are ignored when writing to 0th address
	assign arb_readaddr			= address[7:0] - 8'h1; // reading from this module over avalon from address + 1 reads inner module at 0
	
	always_comb begin
		if (chipselect && read) begin
			if (arb_read)
				readdata = arb_readdata;
			else
				readdata = {23'h0, arb_resultcount, arb_done}; //backwards bc endianness
		end else begin
			readdata = 0;
		end

		if (reset) begin
			arb_reset = 1;
			arb_start = 0;
		end else if (chipselect && write && address == 0) begin
			if (writedata == 0) begin
				arb_start = 0;
				arb_reset = 1;
			end else begin
				arb_reset = 0;
				arb_start = 1;
			end
		end else begin
			arb_start = 0;
			arb_reset = 0;
		end
	end

	bettor_arb #(.MAX_GAME_ID(15), .GAME_ID_WIDTH(4)) mainModule(
		.clk(clk),
		.reset(arb_reset),
		.start(arb_start),
		.write(arb_write),
		.writedata(arb_writedata),
		.readaddress(arb_readaddr),
		.readdata(arb_readdata),
		.done(arb_done),
		.resultcount(arb_resultcount)
	);
endmodule
