module bettor_arb #(parameter MAX_GAME_ID = 1'h1, GAME_ID_WIDTH = 1'h1)
(
	input logic 		clk,
	input logic 		reset,
	input logic 		start,
	input logic			write,
	input logic [31:0]	writedata,
	input logic [7:0]	readaddress,

	output logic		done,
	output logic [7:0]	resultcount,
	output logic [31:0]	readdata
);

	logic [MAX_GAME_ID:0] flushed;
	logic [MAX_GAME_ID:0] found_result;
	logic [MAX_GAME_ID:0] calc_done;
	logic [MAX_GAME_ID:0] calc_write_selector;
	logic [31:0] result [MAX_GAME_ID:0];
	logic [7:0] num_arb;

	// instance array of cal mans share an array of equal length
	// each bit gets assigned to different instance
	// by bitshifting write by the game id, one bit in the selector array
	// is raised: game id 0 is assigned to calc_man[0], game id 6 is assigned
	// to calc_man[6], etc
	assign calc_write_selector = {{MAX_GAME_ID{1'b0}},write} << writedata[23:20];

	// instance array of calc managers
	calc_manager calc_man[MAX_GAME_ID: 0] (
		.clk(clk), // duplicated
		.reset(reset), // duplicated
		.start(start), // duplicated
		.flushed(flushed), // split across instances
		.write(calc_write_selector), // split across instances
		.writedata(writedata),  // duplicated
		.found_result(found_result), // split across instances
		.done(calc_done), // split across instances
		.result(result) // split across instances
	);

	logic [31:0] result_writedata;
	write_manager #(.MAX_GAME_ID(MAX_GAME_ID), .GAME_ID_WIDTH(GAME_ID_WIDTH)) write_man(
		.clk(clk),
		.reset(reset),
		.result_vec(result),
		.found_result_vec(found_result),
		.flushed_vec(flushed),
		.write(write1),
		.write_counter(num_arb),
		.writedata(result_writedata)
	);

	logic write1;
	/* verilator lint_off UNDRIVEN */
	logic write2;
	logic [31:0] writedata2;
	/* verilator lint_on UNDRIVEN */

	/* verilator lint_off UNUSEDSIGNAL */
	logic [31:0] readdata1;
	/* verilator lint_on UNUSEDSIGNAL */

	assign resultcount = num_arb;

	// Port 1 for writing, Port 2 for reading
	twoport_memory result_mem(
		.clk(clk),
		.a1(resultcount),
		.a2(readaddress),
		.write1(write1),
		.write2(write2),
		.writedata1(result_writedata),
		.writedata2(writedata2), 
		.readdata1(readdata1),
		.readdata2(readdata)
	);

	// Introduce one cycle delay to account for writing time
	always_ff @(posedge clk) begin
		done <= calc_done == {MAX_GAME_ID + 1{1'b1}};
	end

endmodule
