module calc_manager(
	input logic			clk,
	input logic			reset,
	input logic			start,
	input logic			flushed,
	/* verilator lint_off UNUSEDSIGNAL */
	input logic			write,
	input logic [31:0]	writedata,
	/* verilator lint_on UNUSEDSIGNAL */

	output logic		found_result,
	output logic		done,
	/* verilator lint_off UNDRIVEN */
	output logic [31:0]	result
	/* verilator lint_on UNDRIVEN */
);

	/* verilator lint_off UNDRIVEN */
	logic [7:0]		e_mem_a1, e_mem_a2;
	logic			e_mem_write1, e_mem_write2;
	logic [31:0]	e_mem_writedata1, e_mem_writedata2;
	/* verilator lint_on UNDRIVEN */

	logic [31:0]	e_mem_readdata1, e_mem_readdata2;

	twoport_memory events_mem(
		.clk(clk),
		.a1(e_mem_a1),
		.write1(e_mem_write1),
		.writedata1(e_mem_writedata1),
		.readdata1(e_mem_readdata1),
		.a2(e_mem_a2),
		.write2(e_mem_write2),
		.writedata2(e_mem_writedata2),
		.readdata2(e_mem_readdata2)
	);


	logic [7:0]	even_index;
	/* verilator lint_off UNUSED */
	logic [7:0] odd_index;
	logic		running;
	/* verilator lint_on UNUSED */

	// events iter should probably just take number of odd and even events
	iterator events_iter(
		.clk(clk),
		.reset(reset),
		.start(start),
		.found_result(found_result),
		.flushed(flushed),
		.even_index_end(next_home_event_index), // after writes stop, these will each be 2 past the last indices written to
		.odd_index_end(next_away_event_index),
		.even_index(even_index),
		.odd_index(odd_index),
		.running(running),
		.done(done)
	);

	logic [31:0]	arb_event_a, arb_event_b;
	logic [19:0]	arb_prob;
	logic			calc_odds_found_result;

	assign found_result = calc_odds_found_result && running;
	calc_odds arb_odds_calc(
		.a(arb_event_a),
		.b(arb_event_b),
		.found_result(calc_odds_found_result),
		.arb_prob(arb_prob)
	);

	logic 		event_is_away;
	logic [7:0] next_home_event_index;
	logic [7:0] next_away_event_index;

	initial begin
		next_home_event_index	= 0;
		next_away_event_index	= 1;
	end 

	assign event_is_away	= writedata[28];
	assign e_mem_writedata1	= writedata;
	assign e_mem_write1		= write;
	assign e_mem_writedata2 = 0;
	assign e_mem_write2		= 0;
	assign e_mem_a2 = odd_index; // port2 always used for reading from odd index

	/* Populate e_mem_a1 with correct idx to write into */
	/* When writes are finished, port1 is switched to reading port */
	always_comb begin
		if (write) begin
			if (event_is_away) e_mem_a1 = next_away_event_index;
			else e_mem_a1 = next_home_event_index;
		end else begin
			e_mem_a1 = even_index;
		end
	end

	// Update indices used for writing
	// Both blocks together: memory[next_index += 2] = writedata;
	always_ff @(posedge clk) begin
		if (reset) begin
			next_home_event_index <= 0;
			next_away_event_index <= 1;
		end
		if (write) begin
			if (event_is_away) next_away_event_index <= next_away_event_index + 2;
			else next_home_event_index <= next_home_event_index + 2;
		end
	end

	// Populate arb_event_a and arb_event_b with correct events for calc_odds to use
	assign arb_event_a = e_mem_readdata1;
	assign arb_event_b = e_mem_readdata2;

	// combinational logic for populating $result
	// TODO would it be crazy to define a module that takes an arb_event_t
	// and exposes the relevant fields?
	assign result = {
		arb_event_b[27:24], // 4 bits of bookie_id_b
		arb_event_a[27:24], // 4 bits of bookie_id_a
		arb_event_a[23:20], // 4 bits of game_id
		arb_prob // 20 bits of arb_prob
	};


endmodule
