module write_manager #(parameter MAX_GAME_ID = 1'h1, GAME_ID_WIDTH = 1'h1)
(
	input logic clk,
	input logic reset,
	input logic [31:0] result_vec [MAX_GAME_ID:0],
	input logic [MAX_GAME_ID:0] found_result_vec,
	output logic [MAX_GAME_ID:0] flushed_vec,
	output logic write,
	output logic [7:0] write_counter,
	output logic [31:0] writedata
); 

	logic [GAME_ID_WIDTH - 1:0] source_index;

	initial begin
		write_counter	= 8'h0;
		source_index	= 0;
		flushed_vec		= 0;
	end

	always_ff @(posedge clk) begin
		if (reset) begin
			write_counter	<= 0;
			flushed_vec		<= 0;
			source_index	<= 0;
		end

		if (source_index == MAX_GAME_ID) begin
			source_index <= 0;
		end else begin
			source_index <= source_index + 1;
		end

		if (found_result_vec[source_index]) begin
			flushed_vec[source_index] <= 1'h1;
		end else begin
			flushed_vec[source_index] <= 1'h0;
		end

		if (found_result_vec[source_index] && !flushed_vec[source_index]) begin
			write_counter <= write_counter + 1'h1;
		end
	end

	always_comb begin
		if (found_result_vec[source_index]) begin
			writedata = result_vec[source_index];
			write = 1'h1;
		end else begin
			writedata = 32'h0;
			write = 1'h0;
		end
	end
endmodule
