#ifndef _ARB_PARSING_H_
#define _ARB_PARSING_H_

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "../include/arb.h"
#include "../include/arb_buf.h"
#include "../include/fixed_point.h"

#define NUM_EVENTS 227

/* Load bets from csv file into temporary struct */
static int parse_csv(const char *filename, struct event_buf *event_buf) {
    FILE* fp;

    // Open csv file for reading
    if ((fp = fopen(filename, "r")) == NULL) {
        perror("fopen()");
        return -1;
    }

    char line[4096];
    int event_idx = 0;

    // Skip header line
    fgets(line, sizeof(line), fp);

    uint32_t game_id;
    char *home;
    uint32_t bookie_id;
    uint32_t outcome;
    char *odds_string;
    double odds;

    // Parse each portion of .csv file
    while (fgets(line, sizeof(line), fp) != NULL) {
        // Parse Game ID, Home Team, Away Team, and Bookie ID
        game_id = atoi(strtok(line, ","));
        home = strtok(NULL, ",");
        strtok(NULL, ",");
        bookie_id = atoi(strtok(NULL, ","));

        // Skip bookie name
        strtok(NULL, ",");

        // Determine outcome based on Outcome Name
        outcome = (strcmp(strtok(NULL, ","), home) == 0) ? 0 : 1;

        // Parse odds
        odds_string = strtok(NULL, ",");
        sscanf(odds_string, "%lf", &odds);


        // Add new event
        arb_event_t new_event = {0};
        init_event(&new_event, double_to_fixed(odds), game_id, bookie_id, outcome);
        event_buf->events_vec[event_idx++] = new_event;
    }

    fclose(fp);
    return event_idx;
}

static struct event_buf *events_from_file(const char *filename)
{
    struct event_buf *events = (struct event_buf *) malloc(sizeof(struct event_buf) + NUM_EVENTS * sizeof(arb_event_t));
    if (events == NULL) {
        perror("malloc");
        exit(1);
    }

    events->len = parse_csv(filename, events);
    if (!events->len) {
        free(events);
        perror("parse csv");
        exit(1);
    }

    printf("%s game odds loaded...\n", filename);

    return events;
}

static void print_arb_results(struct result_buf *arbs)
{
    printf("Arbitrage Opportunities:\n");
    arb_result_t arb;
    double arb_prob;
    for (int i = 0; i < arbs->len; i++) {
        arb = arbs->arbs_vec[i];
        arb_prob = fixed_to_double(arb.arb_prob);
        printf("Game ID: %d, Bookie A: %d, Bookie B: %d, Arbitrage Probability: %f\n", arb.game_id, arb.bookie_id_a, arb.bookie_id_b, arb_prob);
    }
}

#endif
