import time
import csv
import sys

class arb_event:
    def __init__(self, odds, game_id, bookie_id, outcome):
        self.odds = odds
        self.game_id = game_id
        self.bookie_id = bookie_id
        self.outcome = outcome

    def print_arb_event(self):
        print("Outcome: %c" % (self.outcome))
        print("Game ID: %d" % self.game_id)
        print("Bookie ID: %d" % self.bookie_id)
        print("Odds: %f" % self.odds)

class arb_result:
    def __init__(self, arb_prob, game_id, bookie_id_a, bookie_id_b):
        self.arb_prob = arb_prob
        self.game_id = game_id
        self.bookie_id_a = bookie_id_a
        self.bookie_id_b = bookie_id_b

    def print_arb_event(self):
        print("Game ID: %d" % self.game_id)
        print("Bookie ID a: %d" % self.bookie_id_a)
        print("Bookie ID b: %d" % self.bookie_id_b)
        print("Odds: %f" % self.arb_prob)

class event_buf:
    def __init__(self, buf):
        self.events_vec = buf

    def append_event(self, odds, game_id, bookie_id, outcome):
        self.events_vec.append(arb_event(odds, game_id, bookie_id, outcome))

    def print_arb_events(self):
        for event in self.events_vec:
            event.print_arb_event()

class result_buf:
    def __init__(self, buf):
        self.arbs_vec = buf

    def append_result(self, arb_prob, game_id, bookie_id_a, bookie_id_b):
        self.arbs_vec.append(arb_result(arb_prob, game_id, bookie_id_a, bookie_id_b))

    def print_arb_results(self):
        print("Arbitrage Opportunities:\n")
        for result in self.arbs_vec:
            print("Game ID: " + str(result.game_id) + ", Bookie Home: " + str(result.bookie_id_a) + ", Bookie Away: " + str(result.bookie_id_b) + ", Arbitrage Probability: " + str(result.arb_prob))

def calc_arb(events_a: event_buf, events_b: event_buf) -> result_buf:
    results = result_buf([])
    for event_a in events_a.events_vec:
        for event_b in events_b.events_vec:
            if(event_a.game_id != event_b.game_id):
                continue

            if((1 / event_a.odds) + (1 / event_b.odds) < 1):
                results.append_result(event_a.odds + event_b.odds, event_a.game_id, event_a.bookie_id, event_b.bookie_id)

    return results


def read_csv(file_path):
    events_a = event_buf([])
    events_b = event_buf([])

    with open(file_path, mode='r') as csvfile:
        reader = csv.DictReader(csvfile)
        for row in reader:
            game_id = int(row['Game ID'])
            bookie_id = int(row['Bookmaker ID'])
            odds = float(row['Outcome Price'])
            home_team = row['Home Team']
            away_team = row['Away Team']
            outcome_name = row['Outcome Name']

            if outcome_name == home_team:
                outcome = 'a'
                events_a.append_event(odds, game_id, bookie_id, outcome)
            elif outcome_name == away_team:
                outcome = 'b'
                events_b.append_event(odds, game_id, bookie_id, outcome)

    return events_a, events_b


def main(argv):
    filename = 'scrape/nba_game_odds_template.csv'
    if len(argv) == 2:
        filename = argv[1]
    # start_time = time.time()

    # populating events of head to head matchup

    # a = home, b = away
    # events_a, events_b = read_csv('scrape/example_with_arbitrage.csv')
    events_a, events_b = read_csv(filename)

    # print("Outcome a events:")
    # events_a.print_arb_events()
    # print("Outcome b events:")
    # events_b.print_arb_events()

    results = calc_arb(events_a, events_b)

    results.print_arb_results()

    # end_time = time.time()
    # elapsed_time = end_time - start_time
    # print(f"Elapsed time: {elapsed_time:.5f} seconds")

if __name__ == "__main__":
    main(sys.argv)
