import requests
import json
import os
import sys
import csv

# Function to transform and write data to a CSV file
def write_to_csv(transformed_data, file_path):
    with open(file_path, mode='w', newline='') as file:
        writer = csv.writer(file)

        # Write the header row
        writer.writerow(['Game ID', 'Home Team', 'Away Team', 'Bookmaker ID', 'Bookmaker Title', 'Outcome Name', 'Outcome Price'])

        for game in transformed_data:
            game_id = game['id']
            home_team = game['home_team']
            away_team = game['away_team']

            for bookmaker in game['bookmakers']:
                bookmaker_id = bookmaker['key']
                bookmaker_title = bookmaker.get('title', 'N/A')  # Using .get() to handle missing fields gracefully

                for market in bookmaker['markets']:
                    for outcome in market['outcomes']:
                        outcome_name = outcome['name']
                        outcome_price = outcome['price']

                        # Write a row for each outcome
                        writer.writerow([game_id, home_team, away_team, bookmaker_id, bookmaker_title, outcome_name, outcome_price])

def transform_data(data):
    # Dictionary to hold the new integer IDs for bookmakers
    bookie_id_map = {}
    bookie_counter = 0

    for game_index, game in enumerate(data):
        # Redefine the game ID
        game['id'] = game_index

        # Remove unwanted fields
        game.pop('sport_key', None)
        game.pop('sport_title', None)
        game.pop('commence_time', None)

        for bookmaker in game['bookmakers']:
            # Check if bookmaker's key is already mapped, otherwise add to the map
            if bookmaker['key'] not in bookie_id_map:
                bookie_id_map[bookmaker['key']] = bookie_counter
                bookie_counter += 1

            # Redefine the bookmaker's key to the new ID
            bookmaker['key'] = bookie_id_map[bookmaker['key']]

            # Remove unwanted fields
            bookmaker.pop('last_update', None)

            for market in bookmaker['markets']:
                market.pop('key', None)
                market.pop('last_update', None)

    return data

API_KEY = '01a2f84bb8770bcc6ea4494a2d85b5f2'

SPORT = 'basketball_nba'  # use the sport_key from the /sports endpoint below, or use 'upcoming' to see the next 8 games across all sports
REGIONS = 'us'  # uk | us | eu | au. Multiple can be specified if comma delimited
MARKETS = 'h2h'  # h2h | spreads | totals. Multiple can be specified if comma delimited
ODDS_FORMAT = 'decimal'  # decimal | american
DATE_FORMAT = 'iso'  # iso | unix

if __name__ == "__main__":
    odds_response = requests.get(
        'https://api.the-odds-api.com/v4/sports/{}/odds'.format(SPORT),
        params={
            'api_key': API_KEY,
            'regions': REGIONS,
            'markets': MARKETS,
            'oddsFormat': ODDS_FORMAT,
            'dateFormat': DATE_FORMAT,
        }
    )

    if odds_response.status_code != 200:
        print('Failed to get odds: status_code {}, response body {}'.format(odds_response.status_code, odds_response.text))
    else:
        odds_json = odds_response.json()
        transformed_json = transform_data(odds_json)

        csv_file = 'nba_game_odds.csv'
        write_to_csv(transformed_json, os.path.join(sys.path[0], csv_file))

        print('Remaining requests', odds_response.headers['x-requests-remaining'])
        print('Used requests', odds_response.headers['x-requests-used'])
  