#include <iostream>
#include "Varb_avalon_interface.h"
#include <verilated.h>
#include <verilated_vcd_c.h>

#include "../include/arb.h"
#include "../include/arb_buf.h"
#include "../include/arb_parsing.h"
#include "../include/fixed_point.h"
#include "../include/avalon_interface.h"

#define CLOCK_CYCLE 20

void print_result(arb_result_t *result) {
	uint32_t masked_arb_prob = result->arb_prob & 0xFFFFF;
    double arb_prob = fixed_to_double(masked_arb_prob);

    printf("Game ID: %d, Bookie A: %d, Bookie B: %d, Arbitrage Probability: %f\n", result->game_id, result->bookie_id_a, result->bookie_id_b, arb_prob);
}

static void update_clk(Varb_avalon_interface& arbSim, VerilatedVcdC& tfp,
	int time)
{
	arbSim.clk = (time % 20) >= 10;
	arbSim.eval();
	tfp.dump( time );
}

static void avalon_clear_registers(Varb_avalon_interface& arbSim)
{
	arbSim.reset		= 0;
	arbSim.read			= 0;
	arbSim.write		= 0;
	arbSim.chipselect	= 0;
	arbSim.address		= 0;
	arbSim.writedata	= 0;
}

static void avalon_write(Varb_avalon_interface& arbSim, uint32_t data,
	uint16_t addr)
{
	arbSim.reset		= 0;
	arbSim.read			= 0;
	arbSim.write		= 1;
	arbSim.chipselect	= 1;
	arbSim.address		= addr;
	arbSim.writedata	= data;
}

static void avalon_request_read(Varb_avalon_interface& arbSim, uint16_t addr)
{
	arbSim.reset		= 0;
	arbSim.read			= 1;
	arbSim.write		= 0;
	arbSim.chipselect	= 1;
	arbSim.address		= addr;
	arbSim.writedata	= 0;
}

static uint32_t avalon_finish_read(Varb_avalon_interface& arbSim)
{
	uint32_t res = arbSim.readdata;
	avalon_clear_registers(arbSim);
	return res;
}

void run_dummy_avalon_inputs(Varb_avalon_interface& arbSim, VerilatedVcdC& tfp)
{
	int time = 0;
	uint32_t res = 0;

	avalon_clear_registers(arbSim);

	for ( ; time < 500; time += 10) {
		// writing an event
		if (time == 110)
			avalon_write(arbSim, 0xffff, ARB_EVENT_WRITE_ADDR); // all dummy values for now
		if (time == 130)
			avalon_clear_registers(arbSim);

		// asserting start
		if (time == 210)
			avalon_write(arbSim, 0, ARB_START_ADDR);
		if (time == 230)
			avalon_clear_registers(arbSim);

		// checking done + count
		if (time == 310)
			avalon_request_read(arbSim, ARB_READ_REGS_ADDR);
		if (time == 330)
			res = avalon_finish_read(arbSim);

		// reading 0th result
		if (time == 410)
			avalon_request_read(arbSim, ARB_RESULT_READ_ADDR(0));
		if (time == 430)
			res = avalon_finish_read(arbSim);

		update_clk(arbSim, tfp, time);
	}
	update_clk(arbSim, tfp, time);
}

void simulate_actual_events(Varb_avalon_interface& arbSim, VerilatedVcdC& tfp)
{

	int time = 0;
	uint32_t writedata =0;
	uint32_t readdata = 0;
	int event_index = 0;
	struct event_buf *events = events_from_file("../scrape/nba_game_odds_template.csv");
	printf("processing %d events\n", events->len);
	arb_event_t *event = NULL;
	int reset_time = 45;
	int clear_time = -1;
	int write_time = 115;
	int next_polling_ts = -1;
	int complete_poll_ts =-1;
	int clear_start_ts = -1;
	int start_read_ts = -1;
	int finish_read_ts = -1;
	int end_ts = -1;
	uint8_t result_count = 0;
	arb_read_regs_t read_regs = {0};
	arb_result_t *results = NULL;
	uint32_t res = 0;
	uint8_t read_index = 0;

	for ( ; time < 800000; time += 5) {
		// resetting hardware
		if (time == reset_time) {
			avalon_write(arbSim, 0, ARB_RESET_ADDR);
			clear_time = time + CLOCK_CYCLE;
		}

		// writing an event
		if (time == write_time && event_index < events->len) {
			event = &events->events_vec[event_index++];
			writedata = *((uint32_t *)(event));
			avalon_write(arbSim, writedata, ARB_EVENT_WRITE_ADDR);
			clear_time = time + CLOCK_CYCLE;
			write_time += 5 * CLOCK_CYCLE;
		}
		if (time == clear_time) {
			avalon_clear_registers(arbSim);
		}

		// asserting start
		if (time == write_time && event_index == events->len) {
			avalon_write(arbSim, 0, ARB_START_ADDR);
			clear_start_ts = time + CLOCK_CYCLE;
			next_polling_ts = time + 5 * CLOCK_CYCLE;
			clear_time = -1;
		}
		if (time == clear_start_ts) {
			avalon_clear_registers(arbSim);
		}

		// polling for done
		if (time == next_polling_ts) {
			avalon_request_read(arbSim, ARB_READ_REGS_ADDR);
			complete_poll_ts = time + 2 * CLOCK_CYCLE;
		}
		if (time == complete_poll_ts) {
			readdata = avalon_finish_read(arbSim);
			read_regs = *((arb_read_regs_t *) &readdata);
			result_count = read_regs.result_count;
			if (read_regs.done) {
				printf("Done! result count: %d\n", result_count);
				start_read_ts = time + 5 * CLOCK_CYCLE;
				results = (arb_result_t *) malloc(result_count * sizeof(*results));
			} else {
				next_polling_ts += 5 * CLOCK_CYCLE;
			}
		}

		// reading events
		if (time == start_read_ts) {
			avalon_request_read(arbSim, ARB_RESULT_READ_ADDR(read_index));
			finish_read_ts = time + 2 * CLOCK_CYCLE;
		}
		if (time == finish_read_ts) {
			res = avalon_finish_read(arbSim);
			results[read_index] = *((arb_result_t *)&res);
			read_index++;
			if (read_index < result_count)
				start_read_ts = time + 5 * CLOCK_CYCLE;
			else
				end_ts = time + 5 * CLOCK_CYCLE;
		}

		if (time == end_ts) {
			break;
		}

		update_clk(arbSim, tfp, time);
	}
	update_clk(arbSim, tfp, time);

	for (int i = 0; i < result_count; i++) {
		printf("%d: ", i);
		print_result(&results[i]);
	}

	free(events);
	free(results);
}

int main(int argc, const char **argv, const char **env) {
	Verilated::commandArgs(argc, argv);
	Varb_avalon_interface arbSim;
	Verilated::traceEverOn(true);
	VerilatedVcdC tfp;
	arbSim.trace(&tfp, 99);
	tfp.open("arb_avalon_interface.vcd");

	// run_dummy_avalon_inputs(arbSim, tfp);
	simulate_actual_events(arbSim, tfp);


	tfp.close();
	arbSim.final();

	return 0;
}

