#include <iostream>
#include "Vcalc_manager.h"
#include <verilated.h>
#include <verilated_vcd_c.h>

#include "../include/arb.h"
#include "../include/arb_buf.h"
#include "../include/arb_parsing.h"

#define MAX_ENTRIES 256
#define CLOCK_CYCLE 20

void print_result(arb_result_t *result) {
	uint32_t masked_arb_prob = result->arb_prob & 0xFFFFF;
    double arb_prob = fixed_to_double(masked_arb_prob);
	
    printf("Game ID: %d, Bookie A: %d, Bookie B: %d, Arbitrage Probability: %f\n", result->game_id, result->bookie_id_a, result->bookie_id_b, arb_prob);
}

void print_event(arb_event_t *event) {
	printf("odds: %d, bookie_id: %d, game_id: %d, outcome: %d\n", event->odds, event->bookie_id, event->game_id, event->outcome);
}

static void update_clk(Vcalc_manager& calc_man_sim, VerilatedVcdC& tfp,
	int time)
{
	calc_man_sim.clk = (time % 20) >= 10;
	calc_man_sim.eval();
	tfp.dump( time );
}

static void clear_all_regs(Vcalc_manager& calc_man_sim)
{
	calc_man_sim.reset		= 0;
	calc_man_sim.start		= 0;
	calc_man_sim.flushed	= 0;
	calc_man_sim.write		= 0;
	calc_man_sim.writedata	= 0;
}

static void write_one_event(Vcalc_manager& calc_man_sim, arb_event_t *event)
{
	calc_man_sim.write		= 1;
	calc_man_sim.writedata	= *((uint32_t *) event);
}

static void clear_event_writing_regs(Vcalc_manager& calc_man_sim)
{
	calc_man_sim.write		= 0;
	calc_man_sim.writedata	= 0;
}

static void raise_reset(Vcalc_manager& calc_man_sim)
{
	calc_man_sim.reset = 1;
}

static void lower_reset(Vcalc_manager& calc_man_sim)
{
	calc_man_sim.reset = 0;
}

static void raise_start(Vcalc_manager& calc_man_sim)
{
	calc_man_sim.start = 1;
}

static void lower_start(Vcalc_manager& calc_man_sim)
{
	calc_man_sim.start = 0;
}

static void raise_flushed(Vcalc_manager& calc_man_sim)
{
	calc_man_sim.flushed = 1;
}

static void lower_flushed(Vcalc_manager& calc_man_sim)
{
	calc_man_sim.flushed = 0;
}

static int read_found_result(Vcalc_manager& calc_man_sim)
{
	return calc_man_sim.found_result;
}

static uint32_t read_result(Vcalc_manager& calc_man_sim)
{
	return calc_man_sim.result;
}

void simulate_input_events(Vcalc_manager& calc_man_sim, VerilatedVcdC& tfp)
{
	int time = 0;
	int write_index = 0, read_index = 0;
	int write_ts = 40, write_clear_ts = 60;
	int compare_ts = -1;
	int found_result;
	arb_result_t *result;
	arb_event_t *write_event, *read_event;
	struct event_buf *events = events_from_file("../scrape/example_with_arbitrage.csv");
	// struct arb_event_buf *events = events_from_file("../scrape/example_without_arbitrage.csv");
	uint8_t event_count = std::min(MAX_ENTRIES, events->len);
	printf("event_count: %d\n", event_count);
	int reset_time = -1;
	int raise_flush_ts = -1, lower_flush_ts = -1;
	int waiting_for_flush = false;

	for ( ; time < 300000; time += 10) {

		if (time > 460 && calc_man_sim.done) {
			printf("DONE!\n");
			break;
		}
		// (1) Write Phase
		// (a) write next event to memory
		if (time == write_ts && write_index < event_count) {
			write_event = &events->events_vec[write_index];
			write_one_event(calc_man_sim, write_event);
			write_ts += 100;
			write_index++;
		}

		// (b) clear registers
		if (time == write_clear_ts) {
			clear_event_writing_regs(calc_man_sim);
			write_clear_ts += 100;
		}

		// Once all events written, raise start
		if (time == write_ts && write_index >= event_count) {
			printf("<<< all events written >>>\n");
			raise_start(calc_man_sim);
			reset_time = time + CLOCK_CYCLE;
			compare_ts = time + 2 * CLOCK_CYCLE;
		}

		if (time == reset_time) {
			lower_start(calc_man_sim);
		}

		// (2) Comparison phase
		if (time == compare_ts && !waiting_for_flush) {
			found_result = read_found_result(calc_man_sim);
			// uint32_t event_a = read_event_a(calc_man_sim);
			// uint32_t event_b = read_event_b(calc_man_sim);

			// printf("eventA: ");
			// print_event((arb_event_t *) &event_a);
			// printf("eventB: ");
			// print_event((arb_event_t *) &event_b);

			if (found_result) {
				printf("found result!\n");
				uint32_t res = read_result(calc_man_sim);
				raise_flush_ts = time + CLOCK_CYCLE;
				lower_flush_ts = time + 2 * CLOCK_CYCLE;
				result = (arb_result_t *) &res;
				print_result(result);
				waiting_for_flush = true;
			} else {
				compare_ts += CLOCK_CYCLE;
			}
		}

		if (time == raise_flush_ts) {
			raise_flushed(calc_man_sim);
		}

		if (time == lower_flush_ts) {
			lower_flushed(calc_man_sim);
			waiting_for_flush = false;
			compare_ts = time + CLOCK_CYCLE;
		}
		
		update_clk(calc_man_sim, tfp, time);
	}
	update_clk(calc_man_sim, tfp, time);

	free(events);
}

int main(int argc, const char **argv, const char **env) {
	Verilated::commandArgs(argc, argv);
	Vcalc_manager calc_man_sim;
	Verilated::traceEverOn(true);
	VerilatedVcdC tfp;
	calc_man_sim.trace(&tfp, 99);
	tfp.open("calc_manager.vcd");

	simulate_input_events(calc_man_sim, tfp);

	tfp.close();
	calc_man_sim.final();

	return 0;
}

