#include <iostream>
#include "Vcalc_odds.h"
#include <verilated.h>
#include <verilated_vcd_c.h>
#include <cassert>

#include "../include/arb.h"
#include "../include/fixed_point.h"


int main(int argc, const char ** argv, const char ** env) {
	Verilated::commandArgs(argc, argv);

	Vcalc_odds calc_oddsSim;

	assert(sizeof(arb_event_t) * 8 == 32);

	arb_event_t events[6] = {{0}};
	std::string str_odds[6];

	// 50 50
	events[0].odds = double_to_fixed(2.0);
	events[0].outcome = 0;
	events[0].bookie_id = 0;
	str_odds[0] = "50";

	events[1].odds = double_to_fixed(2.0);
	events[1].outcome = 1;
	events[1].bookie_id = 0;
	str_odds[1] = "50";

	// 75 25
	events[2].odds = double_to_fixed(1.33);
	events[2].outcome = 0;
	events[2].bookie_id = 1;
	str_odds[2] = "75";

	events[3].odds = double_to_fixed(4.0);
	events[3].outcome = 1;
	events[3].bookie_id = 1;
	str_odds[3] = "25";

	// 60 40
	events[4].odds = double_to_fixed(1.67);
	events[4].outcome = 0;
	events[4].bookie_id = 2;
	str_odds[4] = "60";

	events[5].odds = double_to_fixed(2.5);
	events[5].outcome = 1;
	events[5].bookie_id = 2;
	str_odds[5] = "40";

	size_t event_count = sizeof(events)/sizeof(events[0]);
	for (int i = 0; i < event_count; i += 2) {
		for (int j = 1; j < event_count; j += 2) {
			if (events[i].outcome == events[j].outcome)
				continue;
			if (events[i].bookie_id == events[j].bookie_id)
				continue;

			std::cout << "Testing " << str_odds[i] << " and " << str_odds[j] << "\n";
			double odds_a = fixed_to_double(events[i].odds);
			double odds_b = fixed_to_double(events[j].odds);

			bool swIsOpp = odds_a + odds_b < odds_a * odds_b;
			auto swArbProb = swIsOpp ? odds_a + odds_b : 0;

			std::cout << "Software Results:\n";
			std::cout << "\tarbitrage: " << (swIsOpp ? "yes" : "no") << "\n";
			std::cout << "\tarbitrage odds: " << swArbProb << "\n";

			calc_oddsSim.a = events[i].odds;
			calc_oddsSim.b = events[j].odds;
			calc_oddsSim.eval();

            bool hwIsArb = calc_oddsSim.found_result;

			// Splice out first 12 bits
			uint32_t maskedArbProb = calc_oddsSim.arb_prob & 0xFFFFF;
			double hwArbProb = fixed_to_double(maskedArbProb);

			// auto hwArbProb = fixed_to_double(calc_oddsSim.arb_prob);

			std::cout << "Hardware Results:\n";
			std::cout << "\tarbitrage: " << (hwIsArb ? "yes" : "no") << "\n";
			std::cout << "\tarbitrage odds: " << hwArbProb << "\n";
			std::cout << "\n";

			assert(hwIsArb == swIsOpp);
		}
	}

	std::cout << "SUCCESS\n";
	calc_oddsSim.final();
}

