#include <iostream>
#include "Viterator.h"
#include <verilated.h>
#include <verilated_vcd_c.h>
#include <cassert>

static void update_clk(Viterator& iteratorSim, VerilatedVcdC& tfp, int time)
{
	iteratorSim.clk = (time % 20) >= 10;
	iteratorSim.eval();
	tfp.dump( time );
}


static void iterator_clear_registers(Viterator& iteratorSim, VerilatedVcdC& tfp) {
    iteratorSim.reset = 0;
    iteratorSim.found_result = 0;
    iteratorSim.flushed = 0;
    iteratorSim.start = 0;
}

static void iterator_reset(Viterator& iteratorSim, VerilatedVcdC& tfp,
    uint8_t even_index_end, uint8_t odd_index_end) {
    iteratorSim.reset = 1;
    iteratorSim.found_result = 0;
    iteratorSim.flushed = 0;
    iteratorSim.even_index_end = even_index_end;
    iteratorSim.odd_index_end = odd_index_end;
}

static void iterator_assert_start(Viterator& iteratorSim, VerilatedVcdC& tfp)
{
    iteratorSim.start = 1;
}

static void iterator_found_result_high(Viterator& iteratorSim, VerilatedVcdC& tfp)
{
    iteratorSim.found_result = 1;
}

static void iterator_found_result_low(Viterator& iteratorSim, VerilatedVcdC& tfp)
{
    iteratorSim.found_result = 0;
}

static void iterator_flushed_high(Viterator& iteratorSim, VerilatedVcdC& tfp)
{
    iteratorSim.flushed = 1;
}

static void iterator_flushed_low(Viterator& iteratorSim, VerilatedVcdC& tfp)
{
    iteratorSim.flushed = 0;
}

static uint8_t iterator_read_even(Viterator& iteratorSim, VerilatedVcdC& tfp) {
    uint8_t res = iteratorSim.even_index;
    return res;
}

static uint8_t iterator_read_dd(Viterator& iteratorSim, VerilatedVcdC& tfp) {
    uint8_t res = iteratorSim.odd_index;
    return res;
}

void simulate_iterator(Viterator& iteratorSim, VerilatedVcdC& tfp)
{
    int time = 0;
    uint8_t even;
    uint8_t odd;

    for ( ; time < 300000; time += 10) {
        /* Pulse reset signal and set num_events */
        if (time == 100)
            iterator_reset(iteratorSim, tfp, 16, 13);
        if (time == 120)
            iterator_clear_registers(iteratorSim, tfp);

        /* Pulse start signal */
        if (time == 140)
            iterator_assert_start(iteratorSim, tfp);
        if (time == 160)
            iterator_clear_registers(iteratorSim, tfp);

        /* Read i and j */
        if (time == 240) {
            even = iterator_read_even(iteratorSim, tfp);
            odd = iterator_read_dd(iteratorSim, tfp);
            printf("-- TIME 240 --:");
            printf("even = %u, odd = %u\n", even, odd);
        }

        if (time == 260) {
            iterator_found_result_high(iteratorSim, tfp);
            even = iterator_read_even(iteratorSim, tfp);
            odd = iterator_read_dd(iteratorSim, tfp);
            printf("-- TIME 260 --:");
            printf("even: %u, odd: %u\n", even, odd);
        }

        if (time == 280) {
            iterator_flushed_high(iteratorSim, tfp);
            even = iterator_read_even(iteratorSim, tfp);
            odd = iterator_read_dd(iteratorSim, tfp);
            printf("-- TIME 280 --:");
            printf("even: %u, odd: %u\n", even, odd);
        }

        if (time == 300) {
            iterator_found_result_low(iteratorSim, tfp);
            even = iterator_read_even(iteratorSim, tfp);
            odd = iterator_read_dd(iteratorSim, tfp);
            printf("-- TIME 300 --:");
            printf("even: %u, odd: %u\n", even, odd);
        }

        if (time == 320) {
            iterator_flushed_low(iteratorSim, tfp);
            even = iterator_read_even(iteratorSim, tfp);
            odd = iterator_read_dd(iteratorSim, tfp);
            printf("-- TIME 320 --:");
            printf("even: %u, odd: %u\n", even, odd);
        }

        if (time == 340) {
            even = iterator_read_even(iteratorSim, tfp);
            odd = iterator_read_dd(iteratorSim, tfp);
            printf("-- TIME 340 --:");
            printf("even: %u, odd: %u\n", even, odd);
        }

        update_clk(iteratorSim, tfp, time);
    }

    update_clk(iteratorSim, tfp, time);
}

int main(int argc, const char ** argv, const char ** env) {
    Verilated::commandArgs(argc, argv);
    Viterator iteratorSim;
    Verilated::traceEverOn(true);
    VerilatedVcdC tfp;
    iteratorSim.trace(&tfp, 99);
    tfp.open("iterator.vcd");

    simulate_iterator(iteratorSim, tfp);

    tfp.close();
    iteratorSim.final();

    return 0;
}