#include <iostream>
#include "Vwrite_manager.h"
#include <verilated.h>
#include <verilated_vcd_c.h>
#include <stdio.h>

#include "../include/arb.h"
#include "../include/arb_buf.h"
#include "../include/arb_parsing.h"


static void update_clk(Vwrite_manager& writeSim, VerilatedVcdC& tfp,
	int time)
{
	writeSim.clk = (time % 20) >= 10;
	writeSim.eval();
	tfp.dump( time );
}


void print_event(arb_event_t *event) {
	printf("odds: %d, bookie_id: %d, game_id: %d, outcome: %d\n", event->odds, event->bookie_id, event->game_id, event->outcome);
}


static void send_result(Vwrite_manager & writeSim, uint32_t data)
{
	uint8_t found_result_vec = 1;
	memcpy(&writeSim.found_result_vec, &found_result_vec,
		sizeof(writeSim.found_result_vec));
	memcpy(&writeSim.result_vec, &data, sizeof(writeSim.result_vec));
}


static void reset_manager(Vwrite_manager& writeSim)
{
	writeSim.reset = 1;	
	memset(&writeSim.found_result_vec, 0, sizeof(writeSim.found_result_vec));
	memset(&writeSim.result_vec, 0, sizeof(writeSim.result_vec));
}


static void clear_registers(Vwrite_manager& writeSim)
{
	writeSim.reset = 0;
	memset(&writeSim.found_result_vec, 0, sizeof(writeSim.found_result_vec));
	memset(&writeSim.result_vec, 0, sizeof(writeSim.result_vec));
}

static uint32_t get_writedata(Vwrite_manager& writeSim)
{
	uint32_t res = writeSim.writedata;
	return res;
}


static void lower_foundresult(Vwrite_manager& writeSim)
{
	writeSim.found_result_vec = 0;
}

static int get_writecounter(Vwrite_manager& writeSim)
{
	return writeSim.write_counter;
}


void simulate_writes(Vwrite_manager& writeSim, VerilatedVcdC& tfp)
{
	int time = 0;
	uint32_t result;

	uint32_t writedata, readdata = 0;
	int event_index = 0;
	struct event_buf *events = events_from_file("../scrape/nba_game_odds_template.csv");
	arb_event_t *event;
	arb_event_t *write_event;
	int write_time = 120;
	int flushed_time = 140;
	int clear_time = 140;
	char wrote_start = 0;

	for ( ; time < 300000; time += 10) {
		if (time == 50)
			reset_manager(writeSim);
		else if (time == 70)
			clear_registers(writeSim);

		// writing an event
		if (time == write_time && event_index < events->len) {
			printf("Sending Event %d:\n", event_index);

			event = &events->events_vec[event_index++];
			result = *((uint32_t *)(event));
			send_result(writeSim, result);
			write_time += 100;

			print_event(event);
		} else if (time == flushed_time) {
			uint32_t res = get_writedata(writeSim);
			write_event = (arb_event_t *) &res;
			flushed_time += 100;

			printf("Written Event: \n");
			print_event(write_event);
		} else if (time + 60 == write_time) {
			lower_foundresult(writeSim);
		}

		if (time == write_time && event_index == events->len)
			break;

		update_clk(writeSim, tfp, time);
	}
	update_clk(writeSim, tfp, time);

	printf("Number of events written: %d\n", get_writecounter(writeSim));
}

int main(int argc, const char **argv, const char **env) {
	Verilated::commandArgs(argc, argv);
	Vwrite_manager writeSim;
	Verilated::traceEverOn(true);
	VerilatedVcdC tfp;
	writeSim.trace(&tfp, 99);
	tfp.open("write_manager.vcd");

	simulate_writes(writeSim, tfp);

	tfp.close();
	writeSim.final();

	return 0;
}

