#include <stdio.h>
#include <sys/ioctl.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <stdint.h>
#include <unistd.h>

#include "../include/arb.h"
#include "../include/arb_ioctl.h"
#include "../include/fixed_point.h"
#include "../include/arb_parsing.h"

void print_arb_event(arb_event_t event)
{
	printf("Outcome: %c\n", OC_IS_A(&event) ? 'a' : 'b');
	printf("Game ID: %d\n", event.game_id);
	printf("Bookie ID: %d\n", event.bookie_id);
	printf("Odds: %f\n", fixed_to_double(event.odds));
        printf("Unused: %d\n", event.unused);
}

void print_arb_result(arb_result_t result)
{
	printf("Game ID: %d\n", result.game_id);
	printf("Bookie ID a: %d\n", result.bookie_id_a);
	printf("Bookie ID b: %d\n", result.bookie_id_b);
	printf("Odds: %f\n", fixed_to_double(result.arb_prob));
}

int main(int argc, char **argv)
{
	if (argc != 2) {
		fprintf(stderr, "usage: %s <events-file>\n", argv[0]);
		exit(1);
	}

	int fd;
    struct result_buf *result_buf = malloc(sizeof(*result_buf) + 256 * sizeof(arb_result_t));

	static const char filename[] = "/dev/calc_arb";
	char *events_file = argv[1];
	struct event_buf *events = events_from_file(events_file);
	
	if((fd = open(filename, O_RDWR)) == -1){
		fprintf(stderr, "could not open %s\n", filename);
		return -1;
  	}

	//call to ioctl
	if (ioctl(fd, CALC_ARB_WRITE_EVENTS, events) == -1){
		perror("ioctl write");
		return -1;
	}

	if (ioctl(fd, CALC_ARB_READ_RESULTS, result_buf) == -1){
		perror("ioctl read");
		return -1;
	}

    printf("Calculations complete...\n");
    print_arb_results(result_buf);

	free(result_buf);
	free(events);

	return 0;
}
